@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -17,6 +17,8 @@
 #
 """ Playing around with constants only. """
 
+# pylint: disable=invalid-name,redefined-outer-name
+
 from __future__ import print_function
 
 try:
@@ -27,10 +29,17 @@ except NameError:
 
 def displayDict(d):
     result = "{"
+    first = True
     for key, value in sorted(d.items()):
-        result += "%s: %s" % (key, value)
+        if not first:
+            result += ","
+
+        result += "%s: %s" % (repr(key), repr(value))
+        first = False
     result += "}"
 
+    return result
+
 
 print("A bunch of constants and their representation:")
 for value in (0, 3, -4, 17, "hey", (0,), 0.0, -0.0):
@@ -106,8 +115,7 @@ mutableConstantChanger()
 
 
 def defaultKeepsIdentity(arg="str_value"):
-    print("Default constant values are still shared if immutable:")
-    print(arg is "str_value")
+    print("Default constant values are still shared if immutable:", arg is "str_value")
 
 
 defaultKeepsIdentity()
