@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -15,7 +15,7 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-""" Reference counting tests.
+""" Reference counting tests for features of Python3.3 or higher.
 
 These contain functions that do specific things, where we have a suspect
 that references may be lost or corrupted. Executing them repeatedly and
@@ -25,6 +25,9 @@ These are Python3 specific constructs, that will give a SyntaxError or
 not be relevant on Python2.
 """
 
+# While we use that for comparison code, no need to compile that.
+# nuitka-project: --nofollow-import-to=nuitka
+
 import os
 import sys
 
@@ -327,8 +330,6 @@ def simpleFunction21():
 
     try:
         gen.throw(GeneratorExit)
-    except GeneratorExit:
-        pass
     except TypeError:
         pass
 
@@ -377,8 +378,8 @@ class ClassIteratorRejectingThrow:
         return next(self.my_iter)
 
     def throw(self, *args):
-        # Some Python3.4 versions do normalize exceptions.
-        assert len(args) == 1 or sys.version_info < (3, 5)
+        # This should not be subject normalize exceptions.
+        assert len(args) == 1, args
 
     __next__ = next
 
@@ -402,6 +403,19 @@ def simpleFunction23():
     gen.throw(MyError)
 
 
+oho = 1
+
+
+def simpleFunction24():
+    def someGenerator():
+        yield from oho
+
+    try:
+        list(someGenerator())
+    except TypeError:
+        pass
+
+
 # These need stderr to be wrapped.
 tests_stderr = (14, 15)
 
