@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -15,7 +15,7 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-""" Reference counting tests.
+""" Reference counting tests for features of Python3.5 or higher.
 
 These contain functions that do specific things, where we have a suspect
 that references may be lost or corrupted. Executing them repeatedly and
@@ -25,6 +25,9 @@ These are Python3.5 specific constructs, that will give a SyntaxError or
 not be relevant on older versions.
 """
 
+# While we use that for comparison code, no need to compile that.
+# nuitka-project: --nofollow-import-to=nuitka
+
 import os
 import sys
 
@@ -38,15 +41,16 @@ sys.path.insert(
 
 # isort:start
 
+# Tests do all bad things:
+# pylint: disable=not-an-iterable
+
+import asyncio
 import types
 
-from nuitka.tools.testing.Common import (
-    checkDebugPython,
-    executeReferenceChecked,
-    run_async,
-)
+from nuitka.PythonVersions import python_version
+from nuitka.tools.testing.Common import executeReferenceChecked, run_async
 
-checkDebugPython()
+# Tests do bad stuff, pylint: disable=redefined-outer-name
 
 
 def raisy():
@@ -145,7 +149,7 @@ class BadAsyncIter:
 
 def simpleFunction7():
     async def someCoroutine():
-        async for i in BadAsyncIter():
+        async for _i in BadAsyncIter():
             print("never going to happen")
 
     try:
@@ -172,12 +176,57 @@ def simpleFunction9():
     return b
 
 
+async def rmtree(path):
+    return await asyncio.get_event_loop().run_in_executor(None, sync_rmtree, path)
+
+
+def sync_rmtree(path):
+    raise FileNotFoundError
+
+
+async def execute():
+    try:
+        await rmtree("/tmp/test1234.txt")
+    except FileNotFoundError:
+        pass
+
+    return 10**10
+
+
+async def run():
+    await execute()
+
+
+def simpleFunction10():
+    asyncio.get_event_loop().run_until_complete(run())
+
+
+def simpleFunction11():
+    async def someCoroutine():
+        return 10
+
+    coro = someCoroutine()
+
+    def someGenerator():
+        yield from coro
+
+    try:
+        list(someGenerator())
+    except TypeError:
+        pass
+
+    coro.close()
+
+
 # These need stderr to be wrapped.
 tests_stderr = ()
 
 # Disabled tests
 tests_skipped = {}
 
+if python_version < 0x380:
+    tests_skipped[10] = "Incompatible refcount bugs of asyncio with python prior 3.8"
+
 result = executeReferenceChecked(
     prefix="simpleFunction",
     names=globals(),
